package writer2latex.rdf;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import org.apache.jena.ontology.OntClass;
import org.apache.jena.ontology.OntModel;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.ModelFactory;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.vocabulary.*;

import writer2latex.xhtml.XhtmlDocument;

import org.apache.jena.rdf.model.Property;
import org.apache.jena.rdf.model.RDFWriter;


public class DocumentStructure {
	private static final String TOCITEM = "TOCItem";
	private HashMap<String, Resource> elements;
	private HashMap<String, DocumentPart> inputParts;
	private final String TS = "https://iph.ras.ru/text_structures#";
	private final String PARSERNAME = "w2phtml"; 
	private final String EXCERPT = "Excerpt";
	private final String TOC_ELEMENT = "TOCElement";
	private final String ELENPHARTICLE = "elenphArticle";
	private String documentID = "DOC_ID";

	private OntModel m;
	private OntClass excerptClass;
	private OntClass elementClass;
	private OntClass itemClass;
	private OntClass elenphClass;
	
	private DocumentStructure() {
		this.elements = new HashMap<String, Resource>();
		this.inputParts = new HashMap<String, DocumentPart>(); 
		this.m = ModelFactory.createOntologyModel();
		this.excerptClass = m.createClass(TS + EXCERPT);
		this.elementClass = m.createClass(TS + TOC_ELEMENT);
		this.itemClass = m.createClass(TS + TOCITEM);
		this.elenphClass = m.createClass(TS + ELENPHARTICLE);


		
	}
	public DocumentStructure(Vector<XhtmlDocument> files) {
		this();
		Iterator<XhtmlDocument> filesIterator = files.iterator();
		while (filesIterator.hasNext()) {
			XhtmlDocument inputDoc = filesIterator.next();
			DocumentPart part = new DocumentPart(inputDoc);
			addPart(part);
		}
		addEmptyParts();
	}
	
	private void addEmptyParts() {
		Set<String> paths = inputParts.keySet();
		String[] array = new String[paths.size()];
		paths.toArray(array);
		for (int k = 0; k < array.length;k++) {
			String[] levels = array[k].split(" ");
			for (int i = 0; i < levels.length; i++) {
				if (levels[i].equals("0")) {
					String emptyPath = createEmptyPath(levels, i);
					if (!inputParts.containsKey(emptyPath)) {
						DocumentPart emptyPart = new DocumentPart(emptyPath);
						addPart(emptyPart);
					}
				}
			}	
		}
	}
	
	private String createEmptyPath(String[] levels, int i) {
		StringBuilder emptyPath = new StringBuilder();
		for (int j = 0; j<= i;j++) {
			if (j != 0) {
				emptyPath.append(" ");
			}
			emptyPath.append(levels[j]);
		}
		return emptyPath.toString();
	}
	public void printModel() {
		m.write(System.out, "RDF/XML-ABBREV");
	}
	
	public void printModel(String fileName) {
		//RDFWriter writer = m.getWriter();
		//writer.setProperty("showXmlDeclaration","true");
		File outFile = new File(fileName + ".rdf");
		FileWriter fw = null;
		try {
			outFile.createNewFile();
			fw = new FileWriter(outFile);
			m.write(fw,"RDF/XML-ABBREV");
			
		} catch (IOException e) {
			System.out.println("File couldn't be created");
			e.printStackTrace();
		} finally {
			try {
				fw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}
	
	
	private void addPart(DocumentPart docExcerpt) {
		inputParts.put(docExcerpt.getPath(), docExcerpt);
	}
	
	private Resource createExcerpt(DocumentPart docExcerpt) {
		String name = TS + EXCERPT + "/" + PARSERNAME + "_" + documentID + docExcerpt.getSafePath();
		Resource excerpt = m.createIndividual(name, excerptClass);
		Property htmlExcerpt = m.createProperty(TS + "htmlExcerpt");
		excerpt.addLiteral(htmlExcerpt, docExcerpt.getBody());

		return excerpt;
	}
	
	private void createElement(DocumentPart docPart) {
		String elementName = TS + TOC_ELEMENT + "/" + PARSERNAME + "_" + documentID + docPart.getSafePath();
		Resource element = m.createIndividual(elementName,elementClass);
		element.addProperty( RDFS.label, docPart.getName());
		elements.put(docPart.getPath(), element);
		attachExcerpt(docPart, element);		
	}	
	
	private void createDocumentElement(DocumentPart docPart) {
		String elementName = TS + ELENPHARTICLE + "/" + PARSERNAME + "_" + documentID ;
		Resource element = m.createResource(elementName,elenphClass);
		element.addProperty( RDFS.label, docPart.getName());

		elements.put(docPart.getPath(), element);
		attachExcerpt(docPart, element);	
	}
	
	private void createTOCItem(DocumentPart docPart) {
		String tocItemName = TS + TOCITEM + "/" + PARSERNAME + "_" + documentID + docPart.getSafePath();
		Resource tocItem = m.createIndividual(tocItemName,itemClass);
		tocItem.addProperty( RDFS.label, docPart.getName());
		Property pointsTo = m.createProperty(TS + "pointsTo");
		Property itemNumber = m.createProperty(TS + "itemNumber");
		Property hasTOCItem = m.createProperty(TS + "hasTOCItem");	
		tocItem.addLiteral(itemNumber, docPart.getNumber());
		

		m.add(tocItem, pointsTo, elements.get(docPart.getPath()));
		if (!docPart.getPath().isEmpty()) {
			Resource parent = elements.get(docPart.getParentPath());
			m.add(parent, hasTOCItem, tocItem);
		}
	
	}
	
	private void attachExcerpt(DocumentPart docPart, Resource element) {
		if (docPart.getBody().isEmpty()) {
			return;
		}
		Resource excerpt = createExcerpt(docPart);
		excerpt.addProperty( RDFS.label, docPart.getName());

		Property hasText = m.createProperty(TS + "hasText");
		element.addProperty(hasText, excerpt);
	}
	
	public void createTree() {
		createElements();
		createTOCItems();
	}
	
	private void createTOCItems() {
		Set<String> paths = inputParts.keySet();
		for (String path : paths) {
			DocumentPart part = inputParts.get(path);
			if (!part.getNumber().equals("")) {
				createTOCItem(part);	
			}
		}		
	}

	private void createElements() {
		Set<String> paths = inputParts.keySet();
		for (String path : paths) {
			DocumentPart part = inputParts.get(path);
			if (part.getPath().isEmpty()) {
				createDocumentElement(part);
			} else {
				createElement(part);
			}
		}
	}

}
